@@ -308,10 +308,18 @@ const ftattr_t	ftattrtab[] = {
 	  FTARG_SIZE, NULL, inobt_crc_flds },
 	{ FLDT_INOBT_SPCRC, "inobt",  NULL, (char *)inobt_spcrc_flds,
 	  btblock_size, FTARG_SIZE, NULL, inobt_spcrc_flds },
+	{ FLDT_FINOBT, "finobt",  NULL, (char *)finobt_flds, btblock_size,
+	  FTARG_SIZE, NULL, finobt_flds },
+	{ FLDT_FINOBT_CRC, "finobt",  NULL, (char *)finobt_crc_flds, btblock_size,
+	  FTARG_SIZE, NULL, finobt_crc_flds },
+	{ FLDT_FINOBT_SPCRC, "finobt",  NULL, (char *)finobt_spcrc_flds,
+	  btblock_size, FTARG_SIZE, NULL, finobt_spcrc_flds },
 	{ FLDT_INOBTKEY, "inobtkey", fp_sarray, (char *)inobt_key_flds,
 	  SI(bitsz(xfs_inobt_key_t)), 0, NULL, inobt_key_flds },
 	{ FLDT_INOBTPTR, "inobtptr", fp_num, "%u", SI(bitsz(xfs_inobt_ptr_t)),
 	  0, fa_agblock, NULL },
+	{ FLDT_FINOBTPTR, "finobtptr", fp_num, "%u", SI(bitsz(xfs_inobt_ptr_t)),
+	  0, fa_agblock, NULL },
 	{ FLDT_INOBTREC, "inobtrec", fp_sarray, (char *)inobt_rec_flds,
 	  SI(bitsz(xfs_inobt_rec_t)), 0, NULL, inobt_rec_flds },
 	{ FLDT_INOBTSPREC, "inobtsprec", fp_sarray, (char *) inobt_sprec_flds,
