@@ -147,8 +147,12 @@ typedef enum fldt	{
 	FLDT_INOBT,
 	FLDT_INOBT_CRC,
 	FLDT_INOBT_SPCRC,
+	FLDT_FINOBT,
+	FLDT_FINOBT_CRC,
+	FLDT_FINOBT_SPCRC,
 	FLDT_INOBTKEY,
 	FLDT_INOBTPTR,
+	FLDT_FINOBTPTR,
 	FLDT_INOBTREC,
 	FLDT_INOBTSPREC,
 	FLDT_INODE,
